// 提供自定义的随机密钥自解密加密算法支持，算法代号：Spiral。
package spiral

import (
	"errors"
	"fmt"
	"strings"

	"archgrid.xyz/ag/toolsbox/encryption"
	"archgrid.xyz/ag/toolsbox/encryption/aes"
	"archgrid.xyz/ag/toolsbox/hash/sha512"
	verifyCode "archgrid.xyz/ag/toolsbox/random/verify_code"
	"archgrid.xyz/ag/toolsbox/serialize/base64"
)

type Strength int

const (
	Compatible Strength = iota
	Enhanced
)

// 根据给定的密钥字符串生成加解密使用的密钥。
func generateKey(key string) []byte {
	keyBytes := sha512.Sha512([]byte(key))
	return keyBytes[4:36]
}

// 对给定的数据进行加密。
func Encrypt(data string, strength ...Strength) (string, error) {
	var ivGen aes.IVGenerator
	if append(strength, Enhanced)[0] == Compatible {
		ivGen = aes.PrefixIVGenerator
	} else {
		ivGen = aes.XorIVGenerator
	}
	key := verifyCode.RandStr(20)
	keyBytes := generateKey(key)
	cipherData, err := aes.Encrypt([]byte(data), keyBytes, encryption.PKCS7Padding, ivGen)
	if err != nil {
		return "", fmt.Errorf("加密计算失败，%w", err)
	}
	var result strings.Builder
	result.WriteString("[")
	result.WriteString(key)
	result.WriteString(base64.ToBase64(cipherData))
	return result.String(), nil
}

// 对给定的数据进行解密。
func Decrypt(data string, strength ...Strength) (string, error) {
	var ivGen aes.IVGenerator
	if append(strength, Enhanced)[0] == Compatible {
		ivGen = aes.PrefixIVGenerator
	} else {
		ivGen = aes.XorIVGenerator
	}
	if message, found := strings.CutPrefix(data, "["); found {
		if len(message) > 20 {
			keySeed := message[:20]
			key := generateKey(keySeed)
			cipherData, err := base64.FromBase64(message[20:])
			if err != nil {
				return "", fmt.Errorf("密文损坏无法解析，%w", err)
			}
			plainText, err := aes.Decrypt(cipherData, key, encryption.PKCS7Padding, ivGen)
			if err != nil {
				return "", fmt.Errorf("密文解密计算失败，%w", err)
			}
			return string(plainText), nil
		}
		return "", errors.New("密文缺损，无法完成解密。")
	}
	return data, nil
}
